@@ -324,14 +324,14 @@ double scaleup(double num, double maximum, double multiplier)
 
 const gpointer int_to_int_pointer(int i)
 {
-    static int max_int = 0;
+    static int array_size = 0;
     static int * array = NULL;
 
-    if ((array == NULL) || (i < max_int)) {
+    if ((array == NULL) || (i > (array_size-1))) {
         array = g_renew(int, array, i+1);
-        while (max_int < i) {
-            array[max_int] = max_int;
-            max_int ++;
+        while (array_size < i+1) {
+            array[array_size] = array_size;
+            array_size ++;
         }
     }
 
@@ -472,7 +472,7 @@ int datasrc_next(void)
 
     if (datasrci < 0) return datasrc_first();
 
-    for (i=(datasrci+1)%limit; (i=i%limit) != datasrci; i++) {
+    for (i=(datasrci+1)%limit; i != datasrci; i=(i+1)%limit) {
         if (datasrc_open(datasrcs[i])) {
             return 1;
         }
